<?php

/* ***********************************************************
 * This file was automatically generated on 2016-02-10.      *
 *                                                           *
 * PHP Bindings Version 2.1.8                                *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

namespace Tinkerforge;

require_once(__DIR__ . '/IPConnection.php');

/**
 * Battery-backed real-time clock
 */
class BrickletRealTimeClock extends Device
{


    /**
     * @internal
     */
    const FUNCTION_SET_DATE_TIME = 1;

    /**
     * @internal
     */
    const FUNCTION_GET_DATE_TIME = 2;

    /**
     * @internal
     */
    const FUNCTION_GET_TIMESTAMP = 3;

    /**
     * @internal
     */
    const FUNCTION_SET_OFFSET = 4;

    /**
     * @internal
     */
    const FUNCTION_GET_OFFSET = 5;

    /**
     * @internal
     */
    const FUNCTION_GET_IDENTITY = 255;

    const WEEKDAY_MONDAY = 1;
    const WEEKDAY_TUESDAY = 2;
    const WEEKDAY_WEDNESDAY = 3;
    const WEEKDAY_THURSDAY = 4;
    const WEEKDAY_FRIDAY = 5;
    const WEEKDAY_SATURDAY = 6;
    const WEEKDAY_SUNDAY = 7;

    const DEVICE_IDENTIFIER = 268;

    const DEVICE_DISPLAY_NAME = "Real-Time Clock Bricklet";

    /**
     * Creates an object with the unique device ID $uid. This object can
     * then be added to the IP connection.
     *
     * @param string $uid
     */
    public function __construct($uid, $ipcon)
    {
        parent::__construct($uid, $ipcon);

        $this->apiVersion = array(2, 0, 0);

        $this->responseExpected[self::FUNCTION_SET_DATE_TIME] = self::RESPONSE_EXPECTED_FALSE;
        $this->responseExpected[self::FUNCTION_GET_DATE_TIME] = self::RESPONSE_EXPECTED_ALWAYS_TRUE;
        $this->responseExpected[self::FUNCTION_GET_TIMESTAMP] = self::RESPONSE_EXPECTED_ALWAYS_TRUE;
        $this->responseExpected[self::FUNCTION_SET_OFFSET] = self::RESPONSE_EXPECTED_FALSE;
        $this->responseExpected[self::FUNCTION_GET_OFFSET] = self::RESPONSE_EXPECTED_ALWAYS_TRUE;
        $this->responseExpected[self::FUNCTION_GET_IDENTITY] = self::RESPONSE_EXPECTED_ALWAYS_TRUE;

    }

    /**
     * @internal
     * @param string $header
     * @param string $data
     */
    public function handleCallback($header, $data)
    {
        call_user_func(array($this, $this->callbackWrappers[$header['functionID']]), $data);
    }

    /**
     * Sets the current date (including weekday) and the current time with hundredths
     * of a second resolution.
     * 
     * Possible value ranges:
     * 
     * * Year: 2000 to 2099
     * * Month: 1 to 12 (January to December)
     * * Day: 1 to 31
     * * Hour: 0 to 23
     * * Minute: 0 to 59
     * * Second: 0 to 59
     * * Centisecond: 0 to 99
     * * Weekday: 1 to 7 (Monday to Sunday)
     * 
     * If the backup battery is installed then the real-time clock keeps date and
     * time even if the Bricklet is not powered by a Brick.
     * 
     * The real-time clock handles leap year and inserts the 29th of February
     * accordingly. But leap seconds, time zones and daylight saving time are not
     * handled.
     * 
     * @param int $year
     * @param int $month
     * @param int $day
     * @param int $hour
     * @param int $minute
     * @param int $second
     * @param int $centisecond
     * @param int $weekday
     * 
     * @return void
     */
    public function setDateTime($year, $month, $day, $hour, $minute, $second, $centisecond, $weekday)
    {
        $payload = '';
        $payload .= pack('v', $year);
        $payload .= pack('C', $month);
        $payload .= pack('C', $day);
        $payload .= pack('C', $hour);
        $payload .= pack('C', $minute);
        $payload .= pack('C', $second);
        $payload .= pack('C', $centisecond);
        $payload .= pack('C', $weekday);

        $this->sendRequest(self::FUNCTION_SET_DATE_TIME, $payload);
    }

    /**
     * Returns the current date (including weekday) and the current time of the
     * real-time clock with hundredths of a second resolution.
     * 
     * 
     * @return array
     */
    public function getDateTime()
    {
        $result = array();

        $payload = '';

        $data = $this->sendRequest(self::FUNCTION_GET_DATE_TIME, $payload);

        $payload = unpack('v1year/C1month/C1day/C1hour/C1minute/C1second/C1centisecond/C1weekday', $data);

        $result['year'] = $payload['year'];
        $result['month'] = $payload['month'];
        $result['day'] = $payload['day'];
        $result['hour'] = $payload['hour'];
        $result['minute'] = $payload['minute'];
        $result['second'] = $payload['second'];
        $result['centisecond'] = $payload['centisecond'];
        $result['weekday'] = $payload['weekday'];

        return $result;
    }

    /**
     * Returns the current date and the time of the real-time clock converted to
     * milliseconds. The timestamp has an effective resolution of hundredths of a
     * second.
     * 
     * 
     * @return int
     */
    public function getTimestamp()
    {
        $payload = '';

        $data = $this->sendRequest(self::FUNCTION_GET_TIMESTAMP, $payload);

        $payload = unpack('C8timestamp', $data);

        return IPConnection::fixUnpackedInt64($payload, 'timestamp');
    }

    /**
     * Sets the offset the real-time clock should compensate for in 2.17 ppm steps
     * between -277.76 ppm (-128) and +275.59 ppm (127).
     * 
     * The real-time clock time can deviate from the actual time due to the frequency
     * deviation of its 32.768 kHz crystal. Even without compensation (factory
     * default) the resulting time deviation should be at most ±20 ppm (±52.6
     * seconds per month).
     * 
     * This deviation can be calculated by comparing the same duration measured by the
     * real-time clock (``rtc_duration``) an accurate reference clock
     * (``ref_duration``).
     * 
     * For best results the configured offset should be set to 0 ppm first and then a
     * duration of at least 6 hours should be measured.
     * 
     * The new offset (``new_offset``) can be calculated from the currently configured
     * offset (``current_offset``) and the measured durations as follow::
     * 
     *   new_offset = current_offset - round(1000000 * (rtc_duration - ref_duration) / rtc_duration / 2.17)
     * 
     * If you want to calculate the offset, then we recommend using the calibration
     * dialog in Brick Viewer, instead of doing it manually.
     * 
     * The offset is saved in the EEPROM of the Bricklet and only needs to be
     * configured once.
     * 
     * @param int $offset
     * 
     * @return void
     */
    public function setOffset($offset)
    {
        $payload = '';
        $payload .= pack('c', $offset);

        $this->sendRequest(self::FUNCTION_SET_OFFSET, $payload);
    }

    /**
     * Returns the offset as set by BrickletRealTimeClock::setOffset().
     * 
     * 
     * @return int
     */
    public function getOffset()
    {
        $payload = '';

        $data = $this->sendRequest(self::FUNCTION_GET_OFFSET, $payload);

        $payload = unpack('c1offset', $data);

        return $payload['offset'];
    }

    /**
     * Returns the UID, the UID where the Bricklet is connected to, 
     * the position, the hardware and firmware version as well as the
     * device identifier.
     * 
     * The position can be 'a', 'b', 'c' or 'd'.
     * 
     * The device identifier numbers can be found :ref:`here <device_identifier>`.
     * |device_identifier_constant|
     * 
     * 
     * @return array
     */
    public function getIdentity()
    {
        $result = array();

        $payload = '';

        $data = $this->sendRequest(self::FUNCTION_GET_IDENTITY, $payload);

        $payload = unpack('c8uid/c8connected_uid/c1position/C3hardware_version/C3firmware_version/v1device_identifier', $data);

        $result['uid'] = IPConnection::implodeUnpackedString($payload, 'uid', 8);
        $result['connected_uid'] = IPConnection::implodeUnpackedString($payload, 'connected_uid', 8);
        $result['position'] = chr($payload['position']);
        $result['hardware_version'] = IPConnection::collectUnpackedArray($payload, 'hardware_version', 3);
        $result['firmware_version'] = IPConnection::collectUnpackedArray($payload, 'firmware_version', 3);
        $result['device_identifier'] = $payload['device_identifier'];

        return $result;
    }
}

?>
